#include "styleoverride.h"
#include <QtGui>
#include <QDebug>

StyleOverride::StyleOverride( QStyle *oldStyle )
    : QStyle()
{
  savedStyle = QStyleFactory::create( oldStyle->objectName() );
  setObjectName( oldStyle->objectName() );
}

StyleOverride::~StyleOverride()
{
  delete savedStyle;
}

QRect StyleOverride::subControlRect(ComplexControl cc, const QStyleOptionComplex *opt,
                             SubControl sc, const QWidget *widget) const
{
  if( cc == CC_SpinBox )
  {
    const QStyleOptionSpinBox *sbox =
        qstyleoption_cast< const QStyleOptionSpinBox * >( opt );
    if( sbox )
    {
      QRect r = sbox->rect;
      qDebug() << widget << r;
      int w = r.height();
      if( r.width() - 2 * w < w )
        w = r.width() / 3;
       switch( sc )
      {
      case SC_SpinBoxUp:
        r.moveLeft( r.right() - w );
        r.setWidth( w );
        return r;
      case SC_SpinBoxDown:
        r.setWidth( w );
        return r;
      case SC_SpinBoxFrame:
        return r;
      case SC_SpinBoxEditField:
        r.moveLeft( w );
        r.setWidth( r.width() - 2 * w );
        r.adjust( 3, 2, -4, -2 );
        return r;
      default:
        break;
      }
    }
  }
  return savedStyle->subControlRect( cc, opt, sc, widget );
}

QStyle::SubControl StyleOverride::hitTestComplexControl(ComplexControl cc, const QStyleOptionComplex *opt,
                                         const QPoint &pt, const QWidget *widget) const
{
  if( cc == CC_SpinBox && opt )
  {
    if( subControlRect( cc, opt, SC_SpinBoxUp ).contains( pt ) )
      return SC_SpinBoxUp;
    else if( subControlRect( cc, opt, SC_SpinBoxDown ).contains( pt ) )
      return SC_SpinBoxDown;
    else if( subControlRect( cc, opt, SC_SpinBoxEditField ).contains( pt ) )
      return SC_SpinBoxEditField;
  }
  return savedStyle->hitTestComplexControl( cc, opt, pt, widget );
}

void StyleOverride::drawComplexControl(ComplexControl cc, const QStyleOptionComplex *opt, QPainter *p,
                                const QWidget *widget) const
{
  if( cc == CC_SpinBox )
  {
    const QStyleOptionSpinBox *sbox =
        qstyleoption_cast< const QStyleOptionSpinBox * >( opt );
    if( sbox )
    {
      QPainter p( ( QWidget * ) widget );

      QStyleOptionButton btn;
      btn.fontMetrics = sbox->fontMetrics;
      btn.palette = sbox->palette;
      btn.state = ( sbox->state | QStyle::State_Raised ) & ~QStyle::State_Sunken;

      // draw up arrow
      btn.rect = subControlRect( CC_SpinBox, opt, SC_SpinBoxUp, widget );
      btn.text = QChar( 0x25B2 );
      if( ( sbox->stepEnabled & QAbstractSpinBox::StepUpEnabled ) == 0 )
      {
        btn.palette.setCurrentColorGroup( QPalette::Disabled );
        btn.state &= ~State_Enabled;
      }
      if( ( sbox->state & QStyle::State_Sunken ) && ( sbox->activeSubControls & SC_SpinBoxUp ) )
        btn.state |= QStyle::State_Sunken;
      drawControl( CE_PushButton, &btn, &p, widget );
      btn.state = ( sbox->state | QStyle::State_Raised ) & ~QStyle::State_Sunken;
      btn.palette.setCurrentColorGroup( sbox->palette.currentColorGroup() );

      // draw down arrow
      btn.rect = subControlRect( CC_SpinBox, opt, SC_SpinBoxDown, widget );
      btn.text = QChar( 0x25BC );
      if( ( sbox->stepEnabled & QAbstractSpinBox::StepDownEnabled ) == 0 )
      {
        btn.palette.setCurrentColorGroup( QPalette::Disabled );
        btn.state &= ~State_Enabled;
      }
      if( ( sbox->state & QStyle::State_Sunken ) && (  sbox->activeSubControls & SC_SpinBoxDown ) )
        btn.state |= QStyle::State_Sunken;
      drawControl( CE_PushButton, &btn, &p, widget );

      // draw edit box border
      QStyleOptionFrameV3 panel;
      panel.fontMetrics = opt->fontMetrics;
      panel.palette = opt->palette;
      panel.rect = subControlRect( CC_SpinBox, opt, SC_SpinBoxEditField, widget )
                   .adjusted( -2, -2, 2, 2 );
      panel.state = opt->state | State_Sunken;
      panel.lineWidth = pixelMetric( PM_DefaultFrameWidth, opt, widget );
      panel.midLineWidth = 0;
      panel.frameShape = QFrame::StyledPanel;
      drawPrimitive( PE_PanelLineEdit, &panel, &p );

      return;
    }
  }
  savedStyle->drawComplexControl( cc, opt, p, widget );
}

