#ifndef STYLEOVERRIDE_H
#define STYLEOVERRIDE_H

#include <QStyle>

class StyleOverride : public QStyle
{
  public:
    StyleOverride(QStyle *oldStyle);
    virtual ~StyleOverride();

    virtual void drawComplexControl(ComplexControl cc, const QStyleOptionComplex *opt, QPainter *p,
                                    const QWidget *widget = 0) const;

    virtual SubControl hitTestComplexControl(ComplexControl cc, const QStyleOptionComplex *opt,
                                             const QPoint &pt, const QWidget *widget = 0) const;

    virtual QRect subControlRect(ComplexControl cc, const QStyleOptionComplex *opt,
                                 SubControl sc, const QWidget *widget = 0) const;


    virtual int pixelMetric(PixelMetric metric, const QStyleOption *option = 0,
                            const QWidget *widget = 0) const
      { return savedStyle->pixelMetric( metric, option, widget ); }

    virtual QRect subElementRect(SubElement subElement, const QStyleOption *option,
                                 const QWidget *widget = 0) const
      { return savedStyle->subElementRect( subElement, option, widget ); }

    virtual QSize sizeFromContents(ContentsType ct, const QStyleOption *opt,
                                   const QSize &contentsSize, const QWidget *w = 0) const
      { return savedStyle->sizeFromContents( ct, opt, contentsSize, w ); }

    virtual void drawPrimitive(PrimitiveElement pe, const QStyleOption *opt, QPainter *p,
                               const QWidget *w = 0) const
      { savedStyle->drawPrimitive( pe, opt, p, w ); }

    virtual void drawControl(ControlElement element, const QStyleOption *opt, QPainter *p,
                             const QWidget *w = 0) const
      { savedStyle->drawControl( element, opt, p, w ); }

    virtual void polish(QWidget *w)
      { savedStyle->polish(w); }

    virtual void unpolish(QWidget *w)
      { savedStyle->unpolish(w); }

    virtual void polish(QApplication *a)
      { savedStyle->polish(a); }

    virtual void unpolish(QApplication *a)
      { savedStyle->unpolish(a); }

    virtual void polish(QPalette &p)
      { savedStyle->polish(p); }

    virtual QRect itemTextRect(const QFontMetrics &fm, const QRect &r,
                               int flags, bool enabled,
                               const QString &text) const
      { return savedStyle->itemTextRect(fm, r, flags, enabled, text); }

    virtual QRect itemPixmapRect(const QRect &r, int flags, const QPixmap &pixmap) const
      { return savedStyle->itemPixmapRect(r, flags, pixmap); }

    virtual void drawItemText(QPainter *painter, const QRect &rect,
                              int flags, const QPalette &pal, bool enabled,
                              const QString &text, QPalette::ColorRole textRole = QPalette::NoRole) const
      { return savedStyle->drawItemText(painter, rect, flags, pal, enabled, text, textRole); }

    virtual void drawItemPixmap(QPainter *painter, const QRect &rect,
                                int alignment, const QPixmap &pixmap) const
      { savedStyle->drawItemPixmap(painter, rect, alignment, pixmap); }

    virtual QPalette standardPalette() const
      { return savedStyle->standardPalette(); }

    virtual int styleHint(StyleHint stylehint, const QStyleOption *opt = 0,
                          const QWidget *widget = 0, QStyleHintReturn* returnData = 0) const
      { return savedStyle->styleHint(stylehint, opt, widget, returnData); }

    virtual QPixmap standardPixmap(StandardPixmap standardPixmap, const QStyleOption *opt = 0,
                                   const QWidget *widget = 0) const
      { return savedStyle->standardPixmap(standardPixmap, opt, widget); }

    QIcon standardIcon(StandardPixmap standardIcon, const QStyleOption *option = 0,
                       const QWidget *widget = 0) const
      { return savedStyle->standardIcon(standardIcon, option, widget); }

    virtual QPixmap generatedIconPixmap(QIcon::Mode iconMode, const QPixmap &pixmap,
                                        const QStyleOption *opt) const
      { return savedStyle->generatedIconPixmap(iconMode, pixmap, opt); }

  private:
    QStyle *savedStyle;
};

#endif // STYLEOVERRIDE_H
